@@ -1,11 +1,21 @@
-from typing import Any, Callable, List, NamedTuple, Tuple, Type
-from ctypes import POINTER, pointer, Structure, HRESULT, c_ulong, c_wchar_p, c_int
 import unittest as ut
+from ctypes import HRESULT, POINTER, Structure, c_int, c_ulong, c_wchar_p, pointer
+from typing import TYPE_CHECKING, Any, Callable, List, NamedTuple, Tuple, Type
 from unittest.mock import MagicMock
 
 import comtypes
-from comtypes.client import IUnknown
-from comtypes._memberspec import _fix_inout_args, _ArgSpecElmType
+from comtypes import IUnknown
+from comtypes._memberspec import _fix_inout_args
+
+if TYPE_CHECKING:
+    from comtypes import hints  # type: ignore
+
+    _ArgSpecType = Tuple[
+        hints.ArgSpecElmType,
+        hints.ArgSpecElmType,
+        hints.ArgSpecElmType,
+        hints.ArgSpecElmType,
+    ]
 
 WSTRING = c_wchar_p
 
@@ -256,7 +266,7 @@ class Test_ArgsKwargsCombinations(ut.TestCase):
 
 
 class PermutedArgspecTestingParams(NamedTuple):
-    argspec: Tuple[_ArgSpecElmType, _ArgSpecElmType, _ArgSpecElmType, _ArgSpecElmType]
+    argspec: "_ArgSpecType"
     args: Tuple[Any, Any, Any]
     orig_ret_val: Tuple[Any, Any, Any]
     fixed_ret_val: List[Any]
