@@ -1,26 +1,80 @@
 import _ctypes
-import ctypes
 import os
 import sys
 import unittest as ut
 import winreg
+from ctypes.wintypes import MAX_PATH
 from unittest import mock
 
 import comtypes
 import comtypes.server.inprocserver
 from comtypes import GUID
 from comtypes.server import register
-from comtypes.server.register import Registrar, RegistryEntries, _get_serverdll
+from comtypes.server.register import (
+    FrozenRegistryEntries,
+    InterpRegistryEntries,
+    Registrar,
+    _delete_key,
+    _get_serverdll,
+)
 
 HKCR = winreg.HKEY_CLASSES_ROOT
 MULTI_SZ = winreg.REG_MULTI_SZ
 SZ = winreg.REG_SZ
 
+ERROR_ACCESS_DENIED = 5
+ERROR_FILE_NOT_FOUND = 2
 
+
+@mock.patch.object(register, "SHDeleteKey")
+@mock.patch.object(register, "winreg")
+class Test_delete_key(ut.TestCase):
+    def test_calls_sh_deletekey_if_forced(self, _winreg, _sh_deletekey):
+        _delete_key(123, "subkey", force=True)
+        _winreg.DeleteKey.assert_not_called()
+        _sh_deletekey.assert_called_once_with(123, "subkey")
+
+    def test_calls_winreg_deletekey_if_not_forced(self, _winreg, _sh_deletekey):
+        _delete_key(123, "subkey", force=False)
+        _winreg.DeleteKey.assert_called_once_with(123, "subkey")
+        _sh_deletekey.assert_not_called()
+
+    def test_ignores_error_on_sh_deletekey(self, _winreg, _sh_deletekey):
+        err = OSError(ERROR_FILE_NOT_FOUND, "msg", "filename", ERROR_FILE_NOT_FOUND)
+        _sh_deletekey.side_effect = err
+        _delete_key(123, "subkey", force=True)
+        _winreg.DeleteKey.assert_not_called()
+        _sh_deletekey.assert_called_once_with(123, "subkey")
+
+    def test_ignores_error_on_winreg_deletekey(self, _winreg, _sh_deletekey):
+        err = OSError(ERROR_FILE_NOT_FOUND, "msg", "filename", ERROR_FILE_NOT_FOUND)
+        _winreg.DeleteKey.side_effect = err
+        _delete_key(123, "subkey", force=False)
+        _winreg.DeleteKey.assert_called_once_with(123, "subkey")
+        _sh_deletekey.assert_not_called()
+
+    def test_not_ignores_winerror_on_sh_deletekey(self, _winreg, _sh_deletekey):
+        err = OSError(ERROR_ACCESS_DENIED, "msg", "filename", ERROR_ACCESS_DENIED)
+        _sh_deletekey.side_effect = err
+        with self.assertRaises(OSError) as e:
+            _delete_key(123, "subkey", force=True)
+        self.assertEqual(e.exception.winerror, ERROR_ACCESS_DENIED)
+        _winreg.DeleteKey.assert_not_called()
+        _sh_deletekey.assert_called_once_with(123, "subkey")
+
+    def test_not_ignores_winerror_on_winreg_deletekey(self, _winreg, _sh_deletekey):
+        err = OSError(ERROR_ACCESS_DENIED, "msg", "filename", ERROR_ACCESS_DENIED)
+        _winreg.DeleteKey.side_effect = err
+        with self.assertRaises(OSError) as e:
+            _delete_key(123, "subkey", force=False)
+        self.assertEqual(e.exception.winerror, ERROR_ACCESS_DENIED)
+        _winreg.DeleteKey.assert_called_once_with(123, "subkey")
+        _sh_deletekey.assert_not_called()
+
+
+@mock.patch.object(register, "winreg")
 class Test_Registrar_nodebug(ut.TestCase):
-    @mock.patch.object(register, "winreg")
     def test_calls_openkey_and_deletekey(self, _winreg):
-        _winreg.HKEY_CLASSES_ROOT = HKCR
         hkey = mock.Mock(spec=winreg.HKEYType)
         _winreg.OpenKey.return_value = hkey
         reg_clsid = GUID.create_new()
@@ -33,10 +87,7 @@ class Test_Registrar_nodebug(ut.TestCase):
         _winreg.OpenKey.assert_called_once_with(HKCR, rf"CLSID\{reg_clsid}")
         _winreg.DeleteKey.assert_called_once_with(hkey, "Logging")
 
-    @mock.patch.object(register, "winreg")
     def test_ignores_winerror(self, _winreg):
-        _winreg.HKEY_CLASSES_ROOT = HKCR
-        ERROR_FILE_NOT_FOUND = 2
         err = OSError(ERROR_FILE_NOT_FOUND, "msg", "filename", ERROR_FILE_NOT_FOUND)
         _winreg.OpenKey.side_effect = err
         reg_clsid = GUID.create_new()
@@ -49,12 +100,9 @@ class Test_Registrar_nodebug(ut.TestCase):
         _winreg.OpenKey.assert_called_once_with(HKCR, rf"CLSID\{reg_clsid}")
         _winreg.DeleteKey.assert_not_called()
 
-    @mock.patch.object(register, "winreg")
     def test_not_ignores_winerror(self, _winreg):
-        _winreg.HKEY_CLASSES_ROOT = HKCR
         hkey = mock.Mock(spec=winreg.HKEYType)
         _winreg.OpenKey.return_value = hkey
-        ERROR_ACCESS_DENIED = 5
         err = OSError(ERROR_ACCESS_DENIED, "msg", "filename", ERROR_ACCESS_DENIED)
         _winreg.OpenKey.return_value = hkey
         _winreg.DeleteKey.side_effect = err
@@ -71,10 +119,9 @@ class Test_Registrar_nodebug(ut.TestCase):
         _winreg.DeleteKey.assert_called_once_with(hkey, "Logging")
 
 
+@mock.patch.object(register, "winreg")
 class Test_Registrar_debug(ut.TestCase):
-    @mock.patch.object(register, "winreg")
     def test_calls_createkey_and_sets_format(self, _winreg):
-        _winreg.HKEY_CLASSES_ROOT = HKCR
         _winreg.REG_MULTI_SZ = MULTI_SZ
         _winreg.REG_SZ = SZ
         hkey = mock.Mock(spec=winreg.HKEYType)
@@ -97,9 +144,7 @@ class Test_Registrar_debug(ut.TestCase):
             ],
         )
 
-    @mock.patch.object(register, "winreg")
     def test_calls_createkey_and_deletes_format(self, _winreg):
-        _winreg.HKEY_CLASSES_ROOT = HKCR
         _winreg.REG_MULTI_SZ = MULTI_SZ
         hkey = mock.Mock(spec=winreg.HKEYType)
         _winreg.CreateKey.return_value = hkey
@@ -117,13 +162,10 @@ class Test_Registrar_debug(ut.TestCase):
         )
         _winreg.DeleteValue.assert_called_once_with(hkey, "format")
 
-    @mock.patch.object(register, "winreg")
     def test_calls_createkey_and_ignores_errors_on_deleting(self, _winreg):
-        _winreg.HKEY_CLASSES_ROOT = HKCR
         _winreg.REG_MULTI_SZ = MULTI_SZ
         hkey = mock.Mock(spec=winreg.HKEYType)
         _winreg.CreateKey.return_value = hkey
-        ERROR_FILE_NOT_FOUND = 2
         err = OSError(ERROR_FILE_NOT_FOUND, "msg", "filename", ERROR_FILE_NOT_FOUND)
         _winreg.DeleteValue.side_effect = err
         levels = ["lv=DEBUG"]
@@ -140,13 +182,10 @@ class Test_Registrar_debug(ut.TestCase):
         )
         _winreg.DeleteValue.assert_called_once_with(hkey, "format")
 
-    @mock.patch.object(register, "winreg")
     def test_calls_createkey_and_not_ignores_errors_on_deleting(self, _winreg):
-        _winreg.HKEY_CLASSES_ROOT = HKCR
         _winreg.REG_MULTI_SZ = MULTI_SZ
         hkey = mock.Mock(spec=winreg.HKEYType)
         _winreg.CreateKey.return_value = hkey
-        ERROR_ACCESS_DENIED = 5
         err = OSError(ERROR_ACCESS_DENIED, "msg", "filename", ERROR_ACCESS_DENIED)
         _winreg.DeleteValue.side_effect = err
         levels = ["lv=DEBUG"]
@@ -191,24 +230,17 @@ class Test_Registrar_unregister(ut.TestCase):
 
 
 class Test_get_serverdll(ut.TestCase):
-    def test_nonfrozen(self):
-        self.assertEqual(_ctypes.__file__, _get_serverdll())
-
-    def test_frozen(self):
-        with mock.patch.object(register, "sys") as _sys:
-            with mock.patch.object(register, "windll") as _windll:
-                handle = 1234
-                _sys.frozendllhandle = handle
-                self.assertEqual(b"\x00" * 260, _get_serverdll())
-                GetModuleFileName = _windll.kernel32.GetModuleFileNameA
-                (((hModule, lpFilename, nSize), _),) = GetModuleFileName.call_args_list
-                self.assertEqual(handle, hModule)
-                buf_type = type(ctypes.create_string_buffer(260))
-                self.assertIsInstance(lpFilename, buf_type)
-                self.assertEqual(260, nSize)
+    @mock.patch.object(register, "GetModuleFileName")
+    def test_frozen(self, GetModuleFileName):
+        handle, dll_path = 1234, r"path\to\frozen.dll"
+        GetModuleFileName.return_value = dll_path
+        self.assertEqual(dll_path, _get_serverdll(handle))
+        (((hmodule, maxsize), _),) = GetModuleFileName.call_args_list
+        self.assertEqual(handle, hmodule)
+        self.assertEqual(MAX_PATH, maxsize)
 
 
-class Test_NonFrozen_RegistryEntries(ut.TestCase):
+class Test_InterpRegistryEntries(ut.TestCase):
     def test_reg_clsid(self):
         reg_clsid = GUID.create_new()
 
@@ -216,7 +248,7 @@ class Test_NonFrozen_RegistryEntries(ut.TestCase):
             _reg_clsid_ = reg_clsid
 
         expected = [(HKCR, rf"CLSID\{reg_clsid}", "", "")]
-        self.assertEqual(expected, list(RegistryEntries(Cls)))
+        self.assertEqual(expected, list(InterpRegistryEntries(Cls)))
 
     def test_reg_desc(self):
         reg_clsid = GUID.create_new()
@@ -227,7 +259,7 @@ class Test_NonFrozen_RegistryEntries(ut.TestCase):
             _reg_desc_ = reg_desc
 
         expected = [(HKCR, rf"CLSID\{reg_clsid}", "", reg_desc)]
-        self.assertEqual(expected, list(RegistryEntries(Cls)))
+        self.assertEqual(expected, list(InterpRegistryEntries(Cls)))
 
     def test_reg_novers_progid(self):
         reg_clsid = GUID.create_new()
@@ -238,7 +270,7 @@ class Test_NonFrozen_RegistryEntries(ut.TestCase):
             _reg_novers_progid_ = reg_novers_progid
 
         expected = [(HKCR, rf"CLSID\{reg_clsid}", "", "Lib Server")]
-        self.assertEqual(expected, list(RegistryEntries(Cls)))
+        self.assertEqual(expected, list(InterpRegistryEntries(Cls)))
 
     def test_progid(self):
         reg_clsid = GUID.create_new()
@@ -254,7 +286,7 @@ class Test_NonFrozen_RegistryEntries(ut.TestCase):
             (HKCR, reg_progid, "", "Lib Server 1"),
             (HKCR, rf"{reg_progid}\CLSID", "", str(reg_clsid)),
         ]
-        self.assertEqual(expected, list(RegistryEntries(Cls)))
+        self.assertEqual(expected, list(InterpRegistryEntries(Cls)))
 
     def test_reg_progid_reg_desc(self):
         reg_clsid = GUID.create_new()
@@ -272,7 +304,7 @@ class Test_NonFrozen_RegistryEntries(ut.TestCase):
             (HKCR, reg_progid, "", "description for testing"),
             (HKCR, rf"{reg_progid}\CLSID", "", str(reg_clsid)),
         ]
-        self.assertEqual(expected, list(RegistryEntries(Cls)))
+        self.assertEqual(expected, list(InterpRegistryEntries(Cls)))
 
     def test_reg_progid_reg_novers_progid(self):
         reg_clsid = GUID.create_new()
@@ -295,7 +327,7 @@ class Test_NonFrozen_RegistryEntries(ut.TestCase):
             (HKCR, rf"{reg_novers_progid}\CurVer", "", "Lib.Server.1"),
             (HKCR, rf"{reg_novers_progid}\CLSID", "", str(reg_clsid)),
         ]
-        self.assertEqual(expected, list(RegistryEntries(Cls)))
+        self.assertEqual(expected, list(InterpRegistryEntries(Cls)))
 
     def test_local_server(self):
         reg_clsid = GUID.create_new()
@@ -311,7 +343,7 @@ class Test_NonFrozen_RegistryEntries(ut.TestCase):
             (HKCR, clsid_sub, "", ""),
             (HKCR, local_srv_sub, "", f"{sys.executable} {__file__}"),
         ]
-        self.assertEqual(expected, list(RegistryEntries(Cls)))
+        self.assertEqual(expected, list(InterpRegistryEntries(Cls)))
 
     def test_inproc_server(self):
         reg_clsid = GUID.create_new()
@@ -330,7 +362,7 @@ class Test_NonFrozen_RegistryEntries(ut.TestCase):
             (HKCR, inproc_srv_sub, "PythonClass", full_classname),
             (HKCR, inproc_srv_sub, "PythonPath", os.path.dirname(__file__)),
         ]
-        self.assertEqual(expected, list(RegistryEntries(Cls)))
+        self.assertEqual(expected, list(InterpRegistryEntries(Cls)))
 
     def test_inproc_server_reg_threading(self):
         reg_clsid = GUID.create_new()
@@ -352,7 +384,7 @@ class Test_NonFrozen_RegistryEntries(ut.TestCase):
             (HKCR, inproc_srv_sub, "PythonPath", os.path.dirname(__file__)),
             (HKCR, inproc_srv_sub, "ThreadingModel", reg_threading),
         ]
-        self.assertEqual(expected, list(RegistryEntries(Cls)))
+        self.assertEqual(expected, list(InterpRegistryEntries(Cls)))
 
     def test_reg_typelib(self):
         reg_clsid = GUID.create_new()
@@ -367,7 +399,7 @@ class Test_NonFrozen_RegistryEntries(ut.TestCase):
             (HKCR, rf"CLSID\{reg_clsid}", "", ""),
             (HKCR, rf"CLSID\{reg_clsid}\Typelib", "", libid),
         ]
-        self.assertEqual(expected, list(RegistryEntries(Cls)))
+        self.assertEqual(expected, list(InterpRegistryEntries(Cls)))
 
     def test_all_entries(self):
         reg_clsid = GUID.create_new()
@@ -408,15 +440,31 @@ class Test_NonFrozen_RegistryEntries(ut.TestCase):
             (HKCR, inproc_srv_sub, "ThreadingModel", reg_threading),
             (HKCR, rf"{clsid_sub}\Typelib", "", libid),
         ]
-        self.assertEqual(expected, list(RegistryEntries(Cls)))
+        self.assertEqual(expected, list(InterpRegistryEntries(Cls)))
 
 
-class Test_Frozen_RegistryEntries(ut.TestCase):
-    @mock.patch.object(register, "sys")
-    def test_local_dll(self, _sys):
-        _sys.mock_add_spec(["executable", "frozen"])
-        _sys.executable = sys.executable
-        _sys.frozen = "dll"
+class Test_FrozenRegistryEntries(ut.TestCase):
+    SERVERDLL = r"my\target\server.dll"
+
+    # We do not test the scenario where `frozen` is `'dll'` but
+    # `frozendllhandle` is `None`, as it is not a situation
+    # we anticipate.
+
+    def test_local_dll(self):
+        reg_clsid = GUID.create_new()
+        reg_clsctx = comtypes.CLSCTX_LOCAL_SERVER
+
+        class Cls:
+            _reg_clsid_ = reg_clsid
+            _reg_clsctx_ = reg_clsctx
+
+        # In such cases, the server does not start because the
+        # InprocServer32/LocalServer32 keys are not registered.
+        expected = [(HKCR, rf"CLSID\{reg_clsid}", "", "")]
+        entries = FrozenRegistryEntries(Cls, frozen="dll", frozendllhandle=1234)
+        self.assertEqual(expected, list(entries))
+
+    def test_local_windows_exe(self):
         reg_clsid = GUID.create_new()
         reg_clsctx = comtypes.CLSCTX_LOCAL_SERVER
 
@@ -424,48 +472,16 @@ class Test_Frozen_RegistryEntries(ut.TestCase):
             _reg_clsid_ = reg_clsid
             _reg_clsctx_ = reg_clsctx
 
-        clsid_sub = rf"CLSID\{reg_clsid}"
         expected = [
-            (HKCR, clsid_sub, "", ""),
-            (HKCR, rf"{clsid_sub}\LocalServer32", "", sys.executable),
+            (HKCR, rf"CLSID\{reg_clsid}", "", ""),
+            (HKCR, rf"CLSID\{reg_clsid}\LocalServer32", "", sys.executable),
         ]
-        self.assertEqual(expected, list(RegistryEntries(Cls)))
+        self.assertEqual(
+            expected, list(FrozenRegistryEntries(Cls, frozen="windows_exe"))
+        )
 
-    @mock.patch.object(register, "sys")
-    def test_local_frozendllhandle(self, _sys):
-        _sys.mock_add_spec(["frozen", "frozendllhandle"])
-        _sys.frozen = "dll"
-        _sys.frozendllhandle = 1234
-        reg_clsid = GUID.create_new()
-        reg_clsctx = comtypes.CLSCTX_LOCAL_SERVER
-
-        class Cls:
-            _reg_clsid_ = reg_clsid
-            _reg_clsctx_ = reg_clsctx
-
-        expected = [(HKCR, rf"CLSID\{reg_clsid}", "", "")]
-        self.assertEqual(expected, list(RegistryEntries(Cls)))
-
-    @mock.patch.object(register, "sys")
-    def test_inproc_windows_exe(self, _sys):
-        _sys.mock_add_spec(["frozen"])
-        _sys.frozen = "windows_exe"
-        reg_clsid = GUID.create_new()
-        reg_clsctx = comtypes.CLSCTX_INPROC_SERVER
-
-        class Cls:
-            _reg_clsid_ = reg_clsid
-            _reg_clsctx_ = reg_clsctx
-
-        expected = [(HKCR, rf"CLSID\{reg_clsid}", "", "")]
-        self.assertEqual(expected, list(RegistryEntries(Cls)))
-
-    @mock.patch.object(register, "_get_serverdll", lambda: r"my\target\server.dll")
-    @mock.patch.object(register, "sys")
-    def test_inproc_dll_frozendllhandle_clsid_to_class(self, _sys):
-        _sys.mock_add_spec(["frozen", "frozendllhandle"])
-        _sys.frozen = "dll"
-        _sys.frozendllhandle = 1234
+    @mock.patch.object(register, "_get_serverdll", return_value=SERVERDLL)
+    def test_inproc_dll(self, get_serverdll):
         reg_clsid = GUID.create_new()
         reg_clsctx = comtypes.CLSCTX_INPROC_SERVER
 
@@ -477,43 +493,15 @@ class Test_Frozen_RegistryEntries(ut.TestCase):
         inproc_srv_sub = rf"{clsid_sub}\InprocServer32"
         expected = [
             (HKCR, clsid_sub, "", ""),
-            (HKCR, inproc_srv_sub, "", r"my\target\server.dll"),
+            (HKCR, inproc_srv_sub, "", self.SERVERDLL),
         ]
 
-        with mock.patch.dict(comtypes.server.inprocserver._clsid_to_class):
-            comtypes.server.inprocserver._clsid_to_class.update({5678: Cls})
-            self.assertEqual(expected, list(RegistryEntries(Cls)))
+        entries = FrozenRegistryEntries(Cls, frozen="dll", frozendllhandle=1234)
+        self.assertEqual(expected, list(entries))
+        get_serverdll.assert_called_once_with(1234)
 
-    @mock.patch.object(register, "_get_serverdll", lambda: r"my\target\server.dll")
-    @mock.patch.object(register, "sys")
-    def test_inproc_dll(self, _sys):
-        _sys.mock_add_spec(["frozen", "modules"])
-        _sys.frozen = "dll"
-        _sys.modules = sys.modules
-        reg_clsid = GUID.create_new()
-        reg_clsctx = comtypes.CLSCTX_INPROC_SERVER
-
-        class Cls:
-            _reg_clsid_ = reg_clsid
-            _reg_clsctx_ = reg_clsctx
-
-        clsid_sub = rf"CLSID\{reg_clsid}"
-        inproc_srv_sub = rf"{clsid_sub}\InprocServer32"
-        full_classname = f"{__name__}.Cls"
-        expected = [
-            (HKCR, clsid_sub, "", ""),
-            (HKCR, inproc_srv_sub, "", r"my\target\server.dll"),
-            (HKCR, inproc_srv_sub, "PythonClass", full_classname),
-            (HKCR, inproc_srv_sub, "PythonPath", os.path.dirname(__file__)),
-        ]
-        self.assertEqual(expected, list(RegistryEntries(Cls)))
-
-    @mock.patch.object(register, "_get_serverdll", lambda: r"my\target\server.dll")
-    @mock.patch.object(register, "sys")
-    def test_inproc_dll_reg_threading(self, _sys):
-        _sys.mock_add_spec(["frozen", "modules"])
-        _sys.frozen = "dll"
-        _sys.modules = sys.modules
+    @mock.patch.object(register, "_get_serverdll", return_value=SERVERDLL)
+    def test_inproc_reg_threading(self, get_serverdll):
         reg_clsid = GUID.create_new()
         reg_threading = "Both"
         reg_clsctx = comtypes.CLSCTX_INPROC_SERVER
@@ -525,12 +513,13 @@ class Test_Frozen_RegistryEntries(ut.TestCase):
 
         clsid_sub = rf"CLSID\{reg_clsid}"
         inproc_srv_sub = rf"{clsid_sub}\InprocServer32"
-        full_classname = f"{__name__}.Cls"
         expected = [
             (HKCR, clsid_sub, "", ""),
-            (HKCR, inproc_srv_sub, "", r"my\target\server.dll"),
-            (HKCR, inproc_srv_sub, "PythonClass", full_classname),
-            (HKCR, inproc_srv_sub, "PythonPath", os.path.dirname(__file__)),
+            (HKCR, inproc_srv_sub, "", self.SERVERDLL),
             (HKCR, inproc_srv_sub, "ThreadingModel", reg_threading),
         ]
-        self.assertEqual(expected, list(RegistryEntries(Cls)))
+        self.assertEqual(
+            expected,
+            list(FrozenRegistryEntries(Cls, frozen="dll", frozendllhandle=1234)),
+        )
+        get_serverdll.assert_called_once_with(1234)
